%%
%% This is file `matlab/dismass.m',
%% generated with the docstrip utility.
%%
%% The original source files were:
%%
%% adjust.dtx  (with options: `dismass')
%% 
%% -------------------------------------------------------------------------
%% Logarithmic binning histogram adjustment.
%% 
%% J. Alex Stark, National Institute of Statistical Sciences, NC.
%% Copyright 1997-9, all rights reserved.  See source file for disclaimer.
%% -------------------------------------------------------------------------
function [hout,dwell,Weight,dwellL,dwellH]=dismass(SScaleT,hin,method,histgen)
%  function [hout, dwell,Weight, dwellL,dwellH] =
%                       dismass(SScaleT, hin, method, histgen)
% 
%  Troublesome SScaleT values include 19.5 and 19.25
% 
%  method is plotting method:
%    'equal':  [default] equal width bins: +/- 0.5 either side
%    'midpt':  logarithmic mid-pt between bin centres
%    'split':  split equal-midpt: equal below threshold if isolated points
%    'hybrid': old method, provided for compatibility
%  split is a compromise of equal and midpt.
% 
%  dwellL and dwellH give lower and upper ranges of histogram dwell
%  times (linear)
% 
%  If a 4th argument is given (of any sort), then all output arguments are
%     constructed for a neat histogram.
% 
%  **** If 4th argument and type equal, then the central dwell times are taken
%       from the non-discrete conversion, that is they are evenly separated.
% 
%  SScaleT =SBins/log10(MaxSampl) where SBins is length(hin)
%  hin is the input histogram assumed to have been binned
%  into SBins equal log width bins
% 
%  For COPYRIGHT information, see the accompanying documentation.
if (nargin<3)
   method='equal';
end
method=lower(method);

[SBins InCols] = size(hin);
if (SBins==1)
  hin=hin';
  [SBins InCols] = size(hin);
end
a_xRand=0.0;

a_x = ceil(10.^(((0:SBins)-0.5)/SScaleT)+...
    (rand(1,SBins+1)-0.5)*a_xRand-1e-10);
tmp = round(SScaleT*log10(a_x))<(0:SBins);
a_x(tmp) = a_x(tmp)+1;
if any(tmp)
  fprintf(2,'Dismass a(x) push ups: %d\n',a_x(tmp));
end
tmp = round(SScaleT*log10(a_x-1+(a_x==1)))>=(0:SBins);
tmp = tmp&(a_x>1);
a_x(tmp) = a_x(tmp)-1;
if any(tmp)
  fprintf(2,'Dismass a(x) push downs: %d\n',a_x(tmp));
end
if ((method(1)=='e') & (nargin>3))
   dwell=10.^(((0:SBins-1)')/SScaleT);
else                % normal situation
   dwell = sqrt(a_x(1:SBins).*(a_x(2:SBins+1)-1))';  % column vector
end
s_x = diff(a_x)';     % column vector
ActiveBins = s_x>0;
NWgt=sum(ActiveBins);
HFac=10.^(0.5/SScaleT);
LFac=10.^(-0.5/SScaleT);

dwellP = dwell(ActiveBins);
if (method(1)=='e')
   dwellH=dwellP*HFac;
   dwellL=dwellP*LFac;
   GapAbove=[s_x([0==1;ActiveBins(1:SBins-1)])==0;0];
elseif (method(1)=='m')
   dwellH = [a_x(2:SBins+1)'-0.5];
   dwellH = dwellH(ActiveBins);
   dwellL = [dwellP(1)*LFac; dwellH(1:NWgt-1)];
elseif (method(1)=='s')
   dwellH = [a_x(2:SBins+1)'-0.5];
   dwellH = dwellH(ActiveBins);
   dwellL = [dwellP(1)-0.5; dwellH(1:NWgt-1)];

   thresh = round(SScaleT*log10(  ceil(1/(10.^(1/SScaleT)-1))-1 ));
   tmp=logical([ones(thresh+1,1);zeros(SBins-thresh-1,1)]);

   GapAbove=tmp(ActiveBins);
   dwellH(GapAbove) = (dwellH(GapAbove)-0.5)*HFac;
   GapBelow = [1;GapAbove(1:NWgt-1)];
   dwellL(GapBelow) = (dwellL(GapBelow)+0.5)*LFac;
elseif (method(1)=='h')
   dwellH = [a_x(2:SBins+1)'-0.5];
   dwellH = dwellH(ActiveBins);
   dwellL = [dwellP(1)*LFac; dwellH(1:NWgt-1)];

   tmp=[s_x(2:SBins);0];
   dwellH(tmp(ActiveBins)==0) = dwellP(tmp(ActiveBins)==0)*HFac;
   GapAbove = tmp(ActiveBins)==0;
   tmp=[0; s_x(1:SBins-1)];
   dwellL(tmp(ActiveBins)==0) = dwellP(tmp(ActiveBins)==0)*LFac;
else
   error('Unknown dwell time range method.');
end
IWgtVec = s_x(ActiveBins)./(dwellH-dwellL)...
   .*(log10(dwellH)-log10(dwellL));
if (InCols==1)                       % if input is a vector
   hout = hin(ActiveBins)./IWgtVec;
else                                 % for matrix input
   hout = hin(ActiveBins,:)./(IWgtVec*ones(1,InCols));
end
if (nargin<4)
   dwell =dwell(ActiveBins);
else
   if ((method(1)~='s') & (method(1)~='h') & (method(1)~='e'))
      GapAbove = [];
      % For 'split' etc. provided above
   end
   Itmp=logical([GapAbove'*0+1; GapAbove']);
   tmpH=dwellH;
   tmpL=dwellL;

   dwellH=[tmpH';tmpL(2:NWgt)' 0];
   dwellH = dwellH(:);
   dwellH = dwellH(Itmp);
   dwellL=[tmpL';tmpH(1:NWgt)'];
   dwellL = dwellL(:);
   dwellL = dwellL(Itmp);
   hout = [hout'; hout'*0];
   hout = hout(:);
   hout = hout(Itmp);
end
if (nargout>2)
   Weight=zeros(size(s_x));
   Weight(ActiveBins)=IWgtVec;
end
%% 
%%  End of file.
%% 
%%
%% End of file `matlab/dismass.m'.
